@@ -53,6 +53,9 @@ void        _wnck_window_destroy (WnckWindow *window);
 
 const char* _wnck_window_get_startup_id (WnckWindow *window);
 
+const char* _wnck_window_get_resource_class (WnckWindow *window);
+const char* _wnck_window_get_resource_name  (WnckWindow *window);
+
 WnckWorkspace* _wnck_workspace_create  (int            number,
 					WnckScreen    *screen);
 void           _wnck_workspace_destroy (WnckWorkspace *space);
