@@ -24,9 +24,9 @@
 #include <stdio.h>
 #include "tasklist.h"
 #include "window.h"
-#include "class-group.h"
 #include "window-action-menu.h"
 #include "workspace.h"
+#include "application.h"
 #include "xutils.h"
 #include "private.h"
 
@@ -38,7 +38,7 @@
  *  Maybe fine tune size_allocate() some more...
  *  Better vertical layout handling
  *  prefs
- *  ellipsizing labels
+ *  ellipsizing lables
  *  support for right-click menu merging w/ bonobo for the applet
  *
  */ 
@@ -66,9 +66,10 @@ typedef struct _WnckTaskClass   WnckTaskClass;
 
 typedef enum
 {
-  WNCK_TASK_CLASS_GROUP,
+  WNCK_TASK_APPLICATION,
   WNCK_TASK_WINDOW,
   WNCK_TASK_STARTUP_SEQUENCE
+
 } WnckTaskType;
 
 struct _WnckTask
@@ -82,8 +83,8 @@ struct _WnckTask
   GtkWidget *label;
 
   WnckTaskType type;
-
-  WnckClassGroup *class_group;
+  
+  WnckApplication *application;
   WnckWindow *window;
 #ifdef HAVE_STARTUP_NOTIFICATION
   SnStartupSequence *startup_sequence;
@@ -92,12 +93,11 @@ struct _WnckTask
   gdouble grouping_score;
 
   GList *windows; /* List of the WnckTask for the window,
-		     if this is a class group */
+		     if this is an application */
   gulong state_changed_tag;
   gulong icon_changed_tag;
   gulong name_changed_tag;
-  gulong class_name_changed_tag;
-  gulong class_icon_changed_tag;
+  gulong app_name_changed_tag;
   
   /* task menu */
   GtkWidget *menu;
@@ -122,19 +122,19 @@ struct _WnckTasklistPrivate
   WnckScreen *screen;
 
   WnckTask *active_task; /* NULL if active window not in tasklist */
-  WnckTask *active_class_group; /* NULL if active window not in tasklist */
+  WnckTask *active_app; /* NULL if active window not in tasklist */
   
   gboolean include_all_workspaces;
   
   /* Calculated by update_lists */
-  GList *class_groups;
   GList *windows;
+  GList *applications;
 
   /* Not handled by update_lists */
   GList *startup_sequences;
   
-  GHashTable *class_group_hash;
   GHashTable *win_hash;
+  GHashTable *app_hash;
   
   GtkTooltips *tooltips;
 
@@ -172,8 +172,8 @@ static void wnck_task_finalize    (GObject       *object);
 
 static WnckTask *wnck_task_new_from_window      (WnckTasklist    *tasklist,
 						 WnckWindow      *window);
-static WnckTask *wnck_task_new_from_class_group (WnckTasklist    *tasklist,
-						 WnckClassGroup  *class_group);
+static WnckTask *wnck_task_new_from_application (WnckTasklist    *tasklist,
+						 WnckApplication *application);
 #ifdef HAVE_STARTUP_NOTIFICATION
 static WnckTask *wnck_task_new_from_startup_sequence (WnckTasklist      *tasklist,
                                                       SnStartupSequence *sequence);
@@ -229,8 +229,7 @@ static void     wnck_tasklist_change_active_task       (WnckTasklist *tasklist,
 static gboolean wnck_tasklist_change_active_timeout    (gpointer data);
 static void     wnck_tasklist_activate_task_window     (WnckTask *task);
 
-static void     wnck_tasklist_update_icon_geometries   (WnckTasklist *tasklist,
-							GList        *visible_tasks);
+static void     wnck_tasklist_update_icon_geometries   (WnckTasklist *tasklist);
 static void     wnck_tasklist_connect_screen           (WnckTasklist *tasklist,
                                                         WnckScreen   *screen);
 static void     wnck_tasklist_disconnect_screen        (WnckTasklist *tasklist);
@@ -341,20 +340,13 @@ wnck_task_finalize (GObject *object)
       task->name_changed_tag = 0;
     }
 
-  if (task->class_name_changed_tag)
+  if (task->app_name_changed_tag)
     {
-      g_signal_handler_disconnect (task->class_group,
-				   task->class_name_changed_tag);
-      task->class_name_changed_tag = 0;
+      g_signal_handler_disconnect (task->application,
+				   task->app_name_changed_tag);
+      task->app_name_changed_tag = 0;
     }
-
-  if (task->class_icon_changed_tag)
-    {
-      g_signal_handler_disconnect (task->class_group,
-				   task->class_icon_changed_tag);
-      task->class_icon_changed_tag = 0;
-    }
-
+  
   if (task->menu)
     {
       gtk_widget_destroy (task->menu);
@@ -373,10 +365,10 @@ wnck_task_finalize (GObject *object)
       task->window = NULL;
     }
 
-  if (task->class_group)
+  if (task->application)
     {
-      g_object_unref (task->class_group);
-      task->class_group = NULL;
+      g_object_unref (task->application);
+      task->application = NULL;
     }
 
 #ifdef HAVE_STARTUP_NOTIFICATION
@@ -436,7 +428,7 @@ wnck_tasklist_init (WnckTasklist *tasklist)
   tasklist->priv->include_all_workspaces = FALSE;
   
   tasklist->priv->win_hash = g_hash_table_new (NULL, NULL);
-  tasklist->priv->class_group_hash = g_hash_table_new (NULL, NULL);
+  tasklist->priv->app_hash = g_hash_table_new (NULL, NULL);
   
   tasklist->priv->grouping = WNCK_TASKLIST_AUTO_GROUP;
   tasklist->priv->grouping_limit = DEFAULT_GROUPING_LIMIT;
@@ -481,15 +473,15 @@ wnck_tasklist_finalize (GObject *object)
    * buttons in container destruction
    */
   g_assert (tasklist->priv->windows == NULL);
-  g_assert (tasklist->priv->class_groups == NULL);
+  g_assert (tasklist->priv->applications == NULL);
   g_assert (tasklist->priv->startup_sequences == NULL);
   /* wnck_tasklist_free_tasks (tasklist); */
   
   g_hash_table_destroy (tasklist->priv->win_hash);
   tasklist->priv->win_hash = NULL;
   
-  g_hash_table_destroy (tasklist->priv->class_group_hash);
-  tasklist->priv->class_group_hash = NULL;
+  g_hash_table_destroy (tasklist->priv->app_hash);
+  tasklist->priv->app_hash = NULL;
   
   if (tasklist->priv->activate_timeout_id != 0)
     gtk_timeout_remove (tasklist->priv->activate_timeout_id);
@@ -674,9 +666,9 @@ wnck_tasklist_layout (GtkAllocation *allocation,
 
 static void
 wnck_tasklist_score_groups (WnckTasklist *tasklist,
-			    GList        *ungrouped_class_groups)
+			    GList        *ungrouped_apps)
 {
-  WnckTask *class_group_task;
+  WnckTask *app_task;
   WnckTask *win_task;
   GList *l, *w;
   const char *first_name = NULL;
@@ -684,15 +676,15 @@ wnck_tasklist_score_groups (WnckTasklist *tasklist,
   int n_same_title;
   double same_window_ratio;
 
-  l = ungrouped_class_groups;
+  l = ungrouped_apps;
   while (l != NULL)
     {
-      class_group_task = WNCK_TASK (l->data);
+      app_task = WNCK_TASK (l->data);
 
-      n_windows = g_list_length (class_group_task->windows);
+      n_windows = g_list_length (app_task->windows);
 
       n_same_title = 0;
-      w = class_group_task->windows;
+      w = app_task->windows;
       while (w != NULL)
 	{
 	  win_task = WNCK_TASK (w->data);
@@ -717,38 +709,38 @@ wnck_tasklist_score_groups (WnckTasklist *tasklist,
        *        XP groups by least used, so we probably want to add
        *        total focused time to this expression.
        */
-      class_group_task->grouping_score = -same_window_ratio * 5 + n_windows;
+      app_task->grouping_score = -same_window_ratio * 5 + n_windows;
 
       l = l->next;
     }
 }
 
 static GList *
-wnck_task_get_highest_scored (GList     *ungrouped_class_groups,
-			      WnckTask **class_group_task_out)
+wnck_task_get_highest_scored (GList     *ungrouped_apps,
+			      WnckTask **app_task_out)
 {
-  WnckTask *class_group_task;
+  WnckTask *app_task;
   WnckTask *best_task = NULL;
   double max_score = -1000000000.0; /* Large negative score */
   GList *l;
  
-  l = ungrouped_class_groups;
+  l = ungrouped_apps;
   while (l != NULL)
     {
-      class_group_task = WNCK_TASK (l->data);
+      app_task = WNCK_TASK (l->data);
 
-      if (class_group_task->grouping_score >= max_score)
+      if (app_task->grouping_score >= max_score)
 	{
-	  max_score = class_group_task->grouping_score;
-	  best_task = class_group_task;
+	  max_score = app_task->grouping_score;
+	  best_task = app_task;
 	}
       
       l = l->next;
     }
 
-  *class_group_task_out = best_task;
+  *app_task_out = best_task;
 
-  return g_list_remove (ungrouped_class_groups, best_task);
+  return g_list_remove (ungrouped_apps, best_task);
 }
 
 static void
@@ -763,7 +755,7 @@ wnck_tasklist_size_request  (GtkWidget      *widget,
   /* int u_width, u_height; */
   GList *l;
   GArray *array;
-  GList *ungrouped_class_groups;
+  GList *ungrouped_apps;
   int n_windows;
   int n_startup_sequences;
   int n_rows;
@@ -771,7 +763,7 @@ wnck_tasklist_size_request  (GtkWidget      *widget,
   int n_grouped_buttons;
   gboolean score_set;
   int val;
-  WnckTask *class_group_task;
+  WnckTask *app_task;
   int lowest_range;
   int grouping_limit;
   
@@ -793,7 +785,7 @@ wnck_tasklist_size_request  (GtkWidget      *widget,
       l = l->next;
     }
 
-  l = tasklist->priv->class_groups;
+  l = tasklist->priv->applications;
   while (l != NULL)
     {
       WnckTask *task = WNCK_TASK (l->data);
@@ -860,7 +852,7 @@ wnck_tasklist_size_request  (GtkWidget      *widget,
   n_windows = g_list_length (tasklist->priv->windows);
   n_startup_sequences = g_list_length (tasklist->priv->startup_sequences);
   n_grouped_buttons = 0;
-  ungrouped_class_groups = g_list_copy (tasklist->priv->class_groups);
+  ungrouped_apps = g_list_copy (tasklist->priv->applications);
   score_set = FALSE;
 
   grouping_limit = MIN (tasklist->priv->grouping_limit,
@@ -886,18 +878,18 @@ wnck_tasklist_size_request  (GtkWidget      *widget,
       lowest_range = val;
     }
 
-  while (ungrouped_class_groups != NULL &&
+  while (ungrouped_apps != NULL &&
 	 tasklist->priv->grouping != WNCK_TASKLIST_NEVER_GROUP)
     {
       if (!score_set)
 	{
-	  wnck_tasklist_score_groups (tasklist, ungrouped_class_groups);
+	  wnck_tasklist_score_groups (tasklist, ungrouped_apps);
 	  score_set = TRUE;
 	}
 
-      ungrouped_class_groups = wnck_task_get_highest_scored (ungrouped_class_groups, &class_group_task);
+      ungrouped_apps = wnck_task_get_highest_scored (ungrouped_apps, &app_task);
 
-      n_grouped_buttons += g_list_length (class_group_task->windows) - 1;
+      n_grouped_buttons += g_list_length (app_task->windows) - 1;
 
       wnck_tasklist_layout (&fake_allocation,
 			    tasklist->priv->max_button_width,
@@ -906,7 +898,7 @@ wnck_tasklist_size_request  (GtkWidget      *widget,
 			    &n_cols, &n_rows);
       if (n_cols != last_n_cols &&
 	  (tasklist->priv->grouping == WNCK_TASKLIST_AUTO_GROUP ||
-	   ungrouped_class_groups == NULL))
+	   ungrouped_apps == NULL))
 	{
 	  val = n_cols * tasklist->priv->max_button_width;
 	  if (val >= lowest_range)
@@ -961,7 +953,7 @@ wnck_tasklist_size_allocate (GtkWidget      *widget,
 {
   GtkAllocation child_allocation;
   WnckTasklist *tasklist;
-  WnckTask *class_group_task;
+  WnckTask *app_task;
   int n_windows;
   int n_startup_sequences;
   GList *l;
@@ -972,7 +964,7 @@ wnck_tasklist_size_allocate (GtkWidget      *widget,
   int n_grouped_buttons;
   int i;
   gboolean score_set;
-  GList *ungrouped_class_groups;
+  GList *ungrouped_apps;
   WnckTask *win_task;
   GList *visible_tasks = NULL;
   int grouping_limit;
@@ -982,7 +974,7 @@ wnck_tasklist_size_allocate (GtkWidget      *widget,
   n_windows = g_list_length (tasklist->priv->windows);
   n_startup_sequences = g_list_length (tasklist->priv->startup_sequences);
   n_grouped_buttons = 0;
-  ungrouped_class_groups = g_list_copy (tasklist->priv->class_groups);
+  ungrouped_apps = g_list_copy (tasklist->priv->applications);
   score_set = FALSE;
 
   grouping_limit = MIN (tasklist->priv->grouping_limit,
@@ -994,27 +986,27 @@ wnck_tasklist_size_allocate (GtkWidget      *widget,
 				       tasklist->priv->max_button_height,
 				       n_startup_sequences + n_windows,
 				       &n_cols, &n_rows);
-  while (ungrouped_class_groups != NULL &&
+  while (ungrouped_apps != NULL &&
 	 ((tasklist->priv->grouping == WNCK_TASKLIST_ALWAYS_GROUP) ||
 	  ((tasklist->priv->grouping == WNCK_TASKLIST_AUTO_GROUP) &&
 	   (button_width < grouping_limit))))
     {
       if (!score_set)
 	{
-	  wnck_tasklist_score_groups (tasklist, ungrouped_class_groups);
+	  wnck_tasklist_score_groups (tasklist, ungrouped_apps);
 	  score_set = TRUE;
 	}
 
-      ungrouped_class_groups = wnck_task_get_highest_scored (ungrouped_class_groups, &class_group_task);
+      ungrouped_apps = wnck_task_get_highest_scored (ungrouped_apps, &app_task);
 
-      n_grouped_buttons += g_list_length (class_group_task->windows) - 1;
+      n_grouped_buttons += g_list_length (app_task->windows) - 1;
 
-      if (g_list_length (class_group_task->windows) > 1)
+      if (g_list_length (app_task->windows) > 1)
 	{
-	  visible_tasks = g_list_prepend (visible_tasks, class_group_task);
+	  visible_tasks = g_list_prepend (visible_tasks, app_task);
 	  
-	  /* Hide all this group's windows */
-	  l = class_group_task->windows;
+	  /* Hide all this apps windows */
+	  l = app_task->windows;
 	  while (l != NULL)
 	    {
 	      win_task = WNCK_TASK (l->data);
@@ -1025,8 +1017,8 @@ wnck_tasklist_size_allocate (GtkWidget      *widget,
 	}
       else
 	{
-	  visible_tasks = g_list_prepend (visible_tasks, class_group_task->windows->data);
-	  gtk_widget_set_child_visible (GTK_WIDGET (class_group_task->button), FALSE);
+	  visible_tasks = g_list_prepend (visible_tasks, app_task->windows->data);
+	  gtk_widget_set_child_visible (GTK_WIDGET (app_task->button), FALSE);
 	}
       
       button_width = wnck_tasklist_layout (allocation,
@@ -1036,14 +1028,14 @@ wnck_tasklist_size_allocate (GtkWidget      *widget,
 					   &n_cols, &n_rows);
     }
 
-  /* Add all ungrouped windows to visible_tasks, and hide their class groups */
-  l = ungrouped_class_groups;
+  /* Add all ungrouped windows to visible_tasks, and hide their apps */
+  l = ungrouped_apps;
   while (l != NULL)
     {
-      class_group_task = WNCK_TASK (l->data);
+      app_task = WNCK_TASK (l->data);
       
-      visible_tasks = g_list_concat (visible_tasks, g_list_copy (class_group_task->windows));
-      gtk_widget_set_child_visible (GTK_WIDGET (class_group_task->button), FALSE);
+      visible_tasks = g_list_concat (visible_tasks, g_list_copy (app_task->windows));
+      gtk_widget_set_child_visible (GTK_WIDGET (app_task->button), FALSE);
       l = l->next;
     }
 
@@ -1078,11 +1070,10 @@ wnck_tasklist_size_allocate (GtkWidget      *widget,
       i++;
       l = l->next;
     }
+  g_list_free (visible_tasks);
 
   /* Update icon geometries. */
-  wnck_tasklist_update_icon_geometries (tasklist, visible_tasks);
-  
-  g_list_free (visible_tasks);
+  wnck_tasklist_update_icon_geometries (tasklist);
   
   GTK_WIDGET_CLASS (tasklist_parent_class)->size_allocate (widget, allocation);
 }
@@ -1141,7 +1132,7 @@ wnck_tasklist_forall (GtkContainer *container,
       (* callback) (task->button, callback_data);
     }
   
-  tmp = tasklist->priv->class_groups;
+  tmp = tasklist->priv->applications;
   while (tmp != NULL)
     {
       WnckTask *task = WNCK_TASK (tmp->data);
@@ -1191,8 +1182,8 @@ wnck_tasklist_remove (GtkContainer   *container,
 	  break;
 	}
     }
-
-  tmp = tasklist->priv->class_groups;
+  
+  tmp = tasklist->priv->applications;
   while (tmp != NULL)
     {
       WnckTask *task = WNCK_TASK (tmp->data);
@@ -1200,10 +1191,10 @@ wnck_tasklist_remove (GtkContainer   *container,
       
       if (task->button == widget)
 	{
-	  g_hash_table_remove (tasklist->priv->class_group_hash,
-			       task->class_group);
-	  tasklist->priv->class_groups =
-	    g_list_remove (tasklist->priv->class_groups,
+	  g_hash_table_remove (tasklist->priv->app_hash,
+			       task->application);
+	  tasklist->priv->applications =
+	    g_list_remove (tasklist->priv->applications,
 			   task);
 
           gtk_widget_unparent (widget);
@@ -1332,7 +1323,7 @@ wnck_tasklist_free_tasks (WnckTasklist *tasklist)
   GList *l;
   
   tasklist->priv->active_task = NULL;
-  tasklist->priv->active_class_group = NULL;
+  tasklist->priv->active_app = NULL;
   
   if (tasklist->priv->windows)
     {
@@ -1350,9 +1341,9 @@ wnck_tasklist_free_tasks (WnckTasklist *tasklist)
   g_assert (tasklist->priv->windows == NULL);
   g_assert (g_hash_table_size (tasklist->priv->win_hash) == 0);
 
-  if (tasklist->priv->class_groups)
+  if (tasklist->priv->applications)
     {
-      l = tasklist->priv->class_groups;
+      l = tasklist->priv->applications;
       while (l != NULL)
 	{
 	  WnckTask *task = WNCK_TASK (l->data);
@@ -1363,8 +1354,8 @@ wnck_tasklist_free_tasks (WnckTasklist *tasklist)
 	  gtk_widget_destroy (task->button);
 	}
     }
-  g_assert (tasklist->priv->class_groups == NULL);
-  g_assert (g_hash_table_size (tasklist->priv->class_group_hash) == 0);
+  g_assert (tasklist->priv->applications == NULL);
+  g_assert (g_hash_table_size (tasklist->priv->app_hash) == 0);
 }
 
 static void
@@ -1372,11 +1363,11 @@ wnck_tasklist_update_lists (WnckTasklist *tasklist)
 {
   GList *windows;
   WnckWindow *win;
-  WnckClassGroup *class_group;
+  WnckApplication *app;
   GList *l;
   WnckWorkspace *active_workspace;
+  WnckTask *app_task;
   WnckTask *win_task;
-  WnckTask *class_group_task;
 
   wnck_tasklist_free_tasks (tasklist);
   
@@ -1401,41 +1392,39 @@ wnck_tasklist_update_lists (WnckTasklist *tasklist)
 	  
 	  gtk_widget_set_parent (win_task->button, GTK_WIDGET (tasklist));
 	  gtk_widget_show (win_task->button);
+	  
+	  app = wnck_window_get_application (win);
+	  app_task = g_hash_table_lookup (tasklist->priv->app_hash, app);
 
-	  /* Class group */
-
-	  class_group = wnck_window_get_class_group (win);
-	  class_group_task = g_hash_table_lookup (tasklist->priv->class_group_hash, class_group);
-
-	  if (class_group_task == NULL)
+	  if (app_task == NULL)
 	    {
-	      class_group_task = wnck_task_new_from_class_group (tasklist, class_group);
-	      gtk_widget_set_parent (class_group_task->button, GTK_WIDGET (tasklist));
-	      gtk_widget_show (class_group_task->button);
+	      app_task = wnck_task_new_from_application (tasklist, app);
+	      gtk_widget_set_parent (app_task->button, GTK_WIDGET (tasklist));
+	      gtk_widget_show (app_task->button);
 	      
-	      tasklist->priv->class_groups = g_list_prepend (tasklist->priv->class_groups,
-							     class_group_task);
-	      g_hash_table_insert (tasklist->priv->class_group_hash, class_group, class_group_task);
+	      tasklist->priv->applications = g_list_prepend (tasklist->priv->applications,
+							     app_task);
+	      g_hash_table_insert (tasklist->priv->app_hash, app, app_task);
 	    }
-
-	  class_group_task->windows = g_list_prepend (class_group_task->windows, win_task);
+	  
+	    app_task->windows = g_list_prepend (app_task->windows, win_task);
 	}
       
       l = l->next;
     }
 
-  /* Sort the class group list */
-  l = tasklist->priv->class_groups;
+  /* Sort the application lists */
+  l = tasklist->priv->applications;
   while (l)
     {
-      class_group_task = WNCK_TASK (l->data);
+      app_task = WNCK_TASK (l->data);
 
-      class_group_task->windows = g_list_sort (class_group_task->windows, wnck_task_compare);
+      app_task->windows = g_list_sort (app_task->windows, wnck_task_compare);
 
       /* so the number of windows in the task gets reset on the
        * task label
        */
-      wnck_task_update_visible_state (class_group_task);
+      wnck_task_update_visible_state (app_task);
       
       
       l = l->next;
@@ -1478,66 +1467,52 @@ wnck_tasklist_change_active_task (WnckTasklist *tasklist, WnckTask *active_task)
 
   if (active_task)
     {
-      active_task = g_hash_table_lookup (tasklist->priv->class_group_hash,
-					 active_task->class_group);
+      active_task = g_hash_table_lookup (tasklist->priv->app_hash,
+					 active_task->application);
 
       if (active_task &&
-	  active_task == tasklist->priv->active_class_group)
+	  active_task == tasklist->priv->active_app)
 	return;
 
-      if (tasklist->priv->active_class_group)
+      if (tasklist->priv->active_app)
 	{
-	  tasklist->priv->active_class_group->really_toggling = TRUE;
-	  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (tasklist->priv->active_class_group->button),
+	  tasklist->priv->active_app->really_toggling = TRUE;
+	  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (tasklist->priv->active_app->button),
 					FALSE);
-	  tasklist->priv->active_class_group->really_toggling = FALSE;
+	  tasklist->priv->active_app->really_toggling = FALSE;
 	}
   
-      tasklist->priv->active_class_group = active_task;
+      tasklist->priv->active_app = active_task;
   
-      if (tasklist->priv->active_class_group)
+      if (tasklist->priv->active_app)
 	{
-	  tasklist->priv->active_class_group->really_toggling = TRUE;
-	  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (tasklist->priv->active_class_group->button),
+	  tasklist->priv->active_app->really_toggling = TRUE;
+	  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (tasklist->priv->active_app->button),
 					TRUE);
-	  tasklist->priv->active_class_group->really_toggling = FALSE;
+	  tasklist->priv->active_app->really_toggling = FALSE;
 	}
     }
 }
 
 static void
-wnck_tasklist_update_icon_geometries (WnckTasklist *tasklist,
-				      GList        *visible_tasks)
+wnck_tasklist_update_icon_geometries (WnckTasklist *tasklist)
 {
 	gint x, y, width, height;
-	GList *l1;
+	GList *list;
 	
-	for (l1 = visible_tasks; l1; l1 = l1->next) {
-		WnckTask *task = WNCK_TASK (l1->data);
+	for (list = tasklist->priv->windows; list; list = list->next) {
+		WnckTask *task = WNCK_TASK (list->data);
 		
 		if (!GTK_WIDGET_REALIZED (task->button))
 			continue;
 
 		gdk_window_get_origin (GTK_BUTTON (task->button)->event_window,
-				       &x, &y);
+					    &x, &y);
 		width = task->button->allocation.width;
 		height = task->button->allocation.height;
 
-		if (task->window)
-			wnck_window_set_icon_geometry (task->window,
-						       x, y, width, height);
-		else {
-			GList *l2;
-
-			for (l2 = task->windows; l2; l2 = l2->next) {
-				WnckTask *win_task = WNCK_TASK (l2->data);
-
-				g_assert (win_task->window);
-
-				wnck_window_set_icon_geometry (win_task->window,
-							       x, y, width, height);
-			}
-		}
+		wnck_window_set_icon_geometry (task->window,
+					       x, y, width, height);
 	}
 }
 
@@ -1756,7 +1731,7 @@ wnck_task_popup_menu (WnckTask *task,
   GtkWidget *image;
   GList *l, *list;
   
-  if (task->class_group == NULL)
+  if (task->application == NULL)
     return;
   
   if (task->menu == NULL)
@@ -1824,6 +1799,8 @@ static void
 wnck_task_button_toggled (GtkButton *button,
 			  WnckTask  *task)
 {
+  WnckWindowState state;
+
   /* Did we really want to change the state of the togglebutton? */
   if (task->really_toggling)
     return;
@@ -1836,7 +1813,7 @@ wnck_task_button_toggled (GtkButton *button,
 
   switch (task->type)
     {
-    case WNCK_TASK_CLASS_GROUP:
+    case WNCK_TASK_APPLICATION:
       wnck_task_popup_menu (task, FALSE);
       break;
     case WNCK_TASK_WINDOW:
@@ -1844,6 +1821,9 @@ wnck_task_button_toggled (GtkButton *button,
 	return;
       
       wnck_tasklist_activate_task_window (task);
+
+      /* FIXME why is this here? WTF? */
+      state = wnck_window_get_state (task->window);
       break;
     case WNCK_TASK_STARTUP_SEQUENCE:
       break;
@@ -1858,19 +1838,15 @@ wnck_task_get_text (WnckTask *task)
   
   switch (task->type)
     {
-    case WNCK_TASK_CLASS_GROUP:
-      name = wnck_class_group_get_name (task->class_group);
-      if (name[0] != 0)
-	return g_strdup_printf ("%s (%d)",
-				name,
-				g_list_length (task->windows));
-      else
-	return g_strdup_printf ("(%d)",
-				g_list_length (task->windows));
+    case WNCK_TASK_APPLICATION:
+      return g_strdup_printf ("%s (%d)",
+                              wnck_application_get_icon_name (task->application),
+                              g_list_length (task->windows));
+      break;
 
     case WNCK_TASK_WINDOW:
       state = wnck_window_get_state (task->window);
-      name = wnck_window_get_name (task->window);
+      name = wnck_window_get_icon_name (task->window);
       
       if (state & WNCK_WINDOW_STATE_MINIMIZED)
 	return g_strdup_printf ("[%s]", name);
@@ -1985,13 +1961,12 @@ wnck_task_get_icon (WnckTask *task)
 
   pixbuf = NULL;
   
-  switch (task->type)
+  switch (task->type)    
     {
-    case WNCK_TASK_CLASS_GROUP:
-      pixbuf = wnck_task_scale_icon (wnck_class_group_get_mini_icon (task->class_group),
-				     FALSE);
+    case WNCK_TASK_APPLICATION:
+      pixbuf =  wnck_task_scale_icon (wnck_application_get_mini_icon (task->application),
+				      FALSE);
       break;
-
     case WNCK_TASK_WINDOW:
       state = wnck_window_get_state (task->window);
 
@@ -2075,21 +2050,19 @@ wnck_task_state_changed (WnckWindow     *window,
   
   if (changed_mask & WNCK_WINDOW_STATE_MINIMIZED)
     {
-      WnckTask *win_task;
+      WnckTask *win_task, *app_task;
 
       win_task = g_hash_table_lookup (tasklist->priv->win_hash,
 						window);
-
       if (win_task)
 	{
-	  WnckTask *class_group_task;
-
 	  wnck_task_update_visible_state (win_task);
 	  
-	  class_group_task = g_hash_table_lookup (tasklist->priv->class_group_hash, win_task->class_group);
+	  app_task = g_hash_table_lookup (tasklist->priv->app_hash,
+					  win_task->application);
 
-	  if (class_group_task)
-	    wnck_task_update_visible_state (class_group_task);
+	  if (app_task)
+	    wnck_task_update_visible_state (app_task);
 	}
     }
     
@@ -2116,18 +2089,8 @@ wnck_task_name_changed (WnckWindow *window,
 }
 
 static void
-wnck_task_class_name_changed (WnckClassGroup *class_group,
-			      gpointer        data)
-{
-  WnckTask *task = WNCK_TASK (data);
-
-  if (task)
-    wnck_task_update_visible_state (task);
-}
-
-static void
-wnck_task_class_icon_changed (WnckClassGroup *class_group,
-			      gpointer        data)
+wnck_task_app_name_changed (WnckApplication *app,
+                            gpointer         data)
 {
   WnckTask *task = WNCK_TASK (data);
 
@@ -2187,10 +2150,11 @@ wnck_task_button_press_event (GtkWidget	      *widget,
 
   switch (task->type)
     {
-    case WNCK_TASK_CLASS_GROUP:
+    case WNCK_TASK_APPLICATION:
       wnck_task_popup_menu (task,
-			    event->button == 3);
+                            event->button == 3);
       return TRUE;
+      break;
 
     case WNCK_TASK_WINDOW:
       if (event->button == 1)
@@ -2324,13 +2288,11 @@ wnck_task_create_widgets (WnckTask *task)
 
   switch (task->type)
     {
-    case WNCK_TASK_CLASS_GROUP:
-      task->class_name_changed_tag = g_signal_connect (G_OBJECT (task->class_group), "name_changed",
-						       G_CALLBACK (wnck_task_class_name_changed), task);
-      task->class_icon_changed_tag = g_signal_connect (G_OBJECT (task->class_group), "icon_changed",
-						       G_CALLBACK (wnck_task_class_icon_changed), task);
+    case WNCK_TASK_APPLICATION:
+      task->app_name_changed_tag = g_signal_connect (G_OBJECT (task->application), "name_changed",
+                                                     G_CALLBACK (wnck_task_app_name_changed), task);
       break;
-
+      
     case WNCK_TASK_WINDOW:
       task->state_changed_tag = g_signal_connect (G_OBJECT (task->window), "state_changed",
                                                   G_CALLBACK (wnck_task_state_changed), task->tasklist);
@@ -2342,9 +2304,6 @@ wnck_task_create_widgets (WnckTask *task)
 
     case WNCK_TASK_STARTUP_SEQUENCE:
       break;
-
-    default:
-      g_assert_not_reached ();
     }
 }
 
@@ -2357,9 +2316,9 @@ draw_dot (GdkWindow *window, GdkGC *lgc, GdkGC *dgc, int x, int y)
 
 
 gboolean
-wnck_task_class_group_expose (GtkWidget        *widget,
-			      GdkEventExpose   *event,
-			      gpointer          data)
+wnck_task_app_expose (GtkWidget        *widget,
+		      GdkEventExpose   *event,
+		      gpointer          data)
 {
   GtkStyle *style;
   GdkGC *lgc, *dgc;
@@ -2382,17 +2341,6 @@ wnck_task_class_group_expose (GtkWidget        *widget,
   return FALSE;
 }
 
-static gint
-compare_class_group_tasks (WnckTask *task1, WnckTask *task2)
-{
-  const char *name1, *name2;
-
-  name1 = wnck_class_group_get_name (task1->class_group);
-  name2 = wnck_class_group_get_name (task2->class_group);
-
-  return g_utf8_collate (name1, name2);
-}
-
 static gint
 wnck_task_compare (gconstpointer  a,
 		   gconstpointer  b)
@@ -2406,12 +2354,9 @@ wnck_task_compare (gconstpointer  a,
   
   switch (task1->type)
     {
-    case WNCK_TASK_CLASS_GROUP:
-      if (task2->type == WNCK_TASK_CLASS_GROUP)
-	return compare_class_group_tasks (task1, task2);
-      else
-	return -1; /* Sort groups before everything else */
-
+    case WNCK_TASK_APPLICATION:
+      xid1_1 = xid1_2 = wnck_application_get_xid (task1->application);
+      break;
     case WNCK_TASK_WINDOW:
       xid1_1 = wnck_window_get_group_leader (task1->window);
       xid1_2 = wnck_window_get_xid (task1->window);
@@ -2423,12 +2368,9 @@ wnck_task_compare (gconstpointer  a,
 
   switch (task2->type)
     {
-    case WNCK_TASK_CLASS_GROUP:
-      if (task1->type == WNCK_TASK_CLASS_GROUP)
-	return compare_class_group_tasks (task1, task2);
-      else
-	return 1; /* Sort groups before everything else */
-
+    case WNCK_TASK_APPLICATION:
+      xid2_1 = xid2_2 = wnck_application_get_xid (task2->application);
+      break;
     case WNCK_TASK_WINDOW:
       xid2_1 = wnck_window_get_group_leader (task2->window);
       xid2_2 = wnck_window_get_xid (task2->window);
@@ -2489,7 +2431,7 @@ wnck_task_new_from_window (WnckTasklist *tasklist,
 
   task->type = WNCK_TASK_WINDOW;
   task->window = g_object_ref (window);
-  task->class_group = g_object_ref (wnck_window_get_class_group (window));
+  task->application = g_object_ref (wnck_window_get_application (window));
   task->tasklist = tasklist;
   
   wnck_task_create_widgets (task);
@@ -2500,25 +2442,25 @@ wnck_task_new_from_window (WnckTasklist *tasklist,
 }
 
 static WnckTask *
-wnck_task_new_from_class_group (WnckTasklist   *tasklist,
-				WnckClassGroup *class_group)
+wnck_task_new_from_application (WnckTasklist    *tasklist,
+				WnckApplication *application)
 {
   WnckTask *task;
 
   task = g_object_new (WNCK_TYPE_TASK, NULL);
 
-  task->type = WNCK_TASK_CLASS_GROUP;
+  task->type = WNCK_TASK_APPLICATION;
+  task->application = g_object_ref (application);
   task->window = NULL;
-  task->class_group = g_object_ref (class_group);
   task->tasklist = tasklist;
-
+  
   wnck_task_create_widgets (task);
-
+  
   g_signal_connect_object (task->button, "expose_event",
-                           G_CALLBACK (wnck_task_class_group_expose),
+                           G_CALLBACK (wnck_task_app_expose),
                            G_OBJECT (task),
                            G_CONNECT_AFTER);
-
+  
   return task;
 }
 
@@ -2532,8 +2474,8 @@ wnck_task_new_from_startup_sequence (WnckTasklist      *tasklist,
   task = g_object_new (WNCK_TYPE_TASK, NULL);
 
   task->type = WNCK_TASK_STARTUP_SEQUENCE;
+  task->application = NULL;
   task->window = NULL;
-  task->class_group = NULL;
   task->startup_sequence = sequence;
   sn_startup_sequence_ref (task->startup_sequence);
   task->tasklist = tasklist;
